@@ -19,11 +19,6 @@ use Carp;
 use LWP::UserAgent;
 use HTTP::Request;
 use Date::Manip;
-Date::Manip::Date_Init("TZ=GMT");
-
-use vars qw($VERSION);
-
-$VERSION = '1.21';
 
 my $CSV_XS_Class = 'Text::CSV_XS';
 my $CSV_PP_Class = 'Text::CSV_PP';
@@ -76,6 +71,7 @@ my @Scalar_Flags = qw(
   granularity
   auto_proxy
   row_filter
+  ua_params
 );
 my $SF_pat = join('|', @Scalar_Flags);
 
@@ -91,7 +87,7 @@ my $HF_pat = join('|', @Hash_Flags);
 sub new {
   my $that  = shift;
   my $class = ref($that) || $that;
-  my(@pass, %parms, $k, $v);
+  my(%parms, $k, $v);
   while (($k,$v) = splice(@_, 0, 2)) {
     if ($k eq 'start_date' || $k eq 'end_date' && $v !~ /^\s*$/) {
       $parms{$k} = __PACKAGE__->date_standardize($v);
@@ -149,8 +145,7 @@ sub new {
   elsif ($parms{auto_proxy}) {
     $ua_params->{env_proxy} = 1 if $ENV{http_proxy};
   }
-  delete $parms{env_proxy};
-  $self->{ua} = LWP::UserAgent->new(%$ua_params);
+  $self->{ua} ||= LWP::UserAgent->new(%$ua_params);
 
   if ($self->granularity !~ /^d/i) {
     $start_date = $self->snap_start_date($start_date);
@@ -410,8 +405,8 @@ sub getter {
           if ($target_mode eq 'split') {
             if (@{$rows->[0]} == 2) {
               foreach (@$rows) {
-                if ($_->[-1] =~ /split\s+(\d+):(\d+)/is) {
-                  splice(@$_, -1, 1, $1, $2);
+                if ($_->[-1] =~ /(split\s+)?(\d+)\D+(\d+)/is) {
+                  splice(@$_, -1, 1, $2, $3);
                 }
               }
             }
@@ -1328,7 +1323,7 @@ Different sites have different formats and different limitations on how
 many quotes are returned for each query. See Finance::QuoteHist::Yahoo
 for an example of how to do this.
 
-For more complicated sites, such as Yahoo, overriding additonal methods
+For more complicated sites, such as Yahoo, overriding additional methods
 might be necessary for dealing with things such as splits and dividends.
 
 =head1 METHODS
