@@ -1,132 +1,72 @@
 package Finance::QuoteHist::Yahoo;
 
 use strict;
-use vars qw(@ISA $VERSION);
+use vars qw(@ISA);
 use Carp;
 
-$VERSION = '1.06';
-
 use Finance::QuoteHist::Generic;
 @ISA = qw(Finance::QuoteHist::Generic);
 
-use HTML::TableExtract 2.07;
 use Date::Manip;
-Date::Manip::Date_Init("TZ=GMT");
 
-# Example for HTML output:
+# https://query1.finance.yahoo.com/v7/finance/download/IBM?period1=1495391410&period2=1498069810&interval=1d&events=history&crumb=bB6k340lPXt
+# https://query1.finance.yahoo.com/v7/finance/download/IBM?period1=993096000&period2=1498017600&interval=1wk&events=history&crumb=bB6k340lPXt
+# https://query1.finance.yahoo.com/v7/finance/download/IBM?period1=993096000&period2=1498017600&interval=1mo&events=history&crumb=bB6k340lPXt
 #
-# http://finance.yahoo.com/q/hp?s=IBM&a=00&b=2&c=1800&d=04&e=8&f=2005&g=d&z=66&y=66
+# Dividends:
+# https://query1.finance.yahoo.com/v7/finance/download/IBM?period1=993096000&period2=1498017600&interval=1d&events=div&crumb=bB6k340lPXt
 #
-#   * s - ticker symbol
-#   * a - start month
-#   * b - start day
-#   * c - start year
-#   * d - end month
-#   * e - end day
-#   * f - end year
-#   * g - resolution (e.g. 'd' is daily, 'w' is weekly, 'm' is monthly)
-#   * y is the offset (cursor) from the start date
-#   * z is the number of results to return starting at the cursor (66
-#     maximum, apparently)
-#
-# Note alternate url:
-# http://table.finance.yahoo.com/d?a=1&b=1&c=1800&d=3&e=1&f=2006&s=yhoo&y=200&g=d
-#
-# Example for CSV output:
-#
-# http://ichart.finance.yahoo.com/table.csv?s=IBM&a=00&b=2&c=1800&d=04&e=8&f=2005&g=d&ignore=.csv
-#
-# (historically could use table.finance.yahoo.com as well)
-#
-# Note that Yahoo implements month numbering with Jan=0 and Dec=11.
-#
-# For CSV output, date ranges are unlimited; the output is adjusted and
-# does not include any split or dividend notices.
-#
-# URL for splits and dividends:
-#
-# These are either extracted from within the historical quote results
-# (non_quote_row()) or found directly:
-#
-# http://finance.yahoo.com/q/hp?s=IBM&a=00&b=2&c=1800&d=04&e=8&f=2005&g=v
-# http://table.finance.yahoo.com/table.csv?s=IBM&a=00&b=2&c=1800&d=04&e=8&f=2005&g=v&ignore=.csv
-#
-# Example URL for weekly:
-#
-# http://finance.yahoo.com/q/hp?s=IBM&a=00&b=2&c=1962&d=01&e=27&f=2006&g=w
-#
-# Example URL for monthly:
-#
-# http://finance.yahoo.com/q/hp?s=IBM&a=00&b=2&c=1962&d=01&e=27&f=2006&g=m
+# Splits:
+# https://query1.finance.yahoo.com/v7/finance/download/NKE?period1=993096000&period2=1498017600&interval=1d&events=split&crumb=bB6k340lPXt
 
 sub new {
   my $that = shift;
   my $class = ref($that) || $that;
   my %parms = @_;
 
-  $parms{parse_mode} ||= 'csv';
+  $parms{parse_mode} = 'csv';
+  $parms{ua_params} ||= {};
+  $parms{ua_params}{cookie_jar} ||= {};
 
   my $self = __PACKAGE__->SUPER::new(%parms);
   bless $self, $class;
 
+  # set initial cookie (the cookie crumbs are hashed out of this)
+  # https://finance.yahoo.com/quote/IBM/history
+  my $ticker = $parms{symbols};
+  $ticker = $ticker->[0] if ref $ticker eq 'ARRAY';
+  my $html = $self->fetch("https://finance.yahoo.com/quote/$ticker/history");
+
+  # extract the cookie crumb
+  my %crumbs;
+  for my $c ($html =~ /"crumb"\s*:\s*"([^"]+)"/g) {
+    next if $c =~ /[{}]/;
+    $c =~ s/\\u002F/\//;
+    ++$crumbs{$c};
+  }
+  my $crumb = '';
+  my $max = 0;
+  for my $c (keys %crumbs) {
+    if ($crumbs{$c} >= $max) {
+      $crumb = $c;
+      $max = $crumbs{$c};
+    }
+  }
+
+  $self->{crumb} = $crumb;
+
   $self;
 }
 
-# override these for other Yahoo sites
-sub url_base_csv    { 'http://ichart.finance.yahoo.com/table.csv' }
-sub url_base_html   { 'http://finance.yahoo.com/q/hp' }
-
 sub granularities { qw( daily weekly monthly ) }
 
-# Yahoo can fetch dividends and splits. They can be extracted from
-# regular quote results or queried directly.
-
-#sub html_parser {
-#  my($self, %parms) = @_;
-#  my $target_mode = $parms{target_mode} || $self->target_mode;
-#  return $self->SUPER::html_parser(%parms) unless $target_mode eq 'split';
-#  sub {
-#    my $data = shift;
-#    my $html_string;
-#    if (ref $data) {
-#      local($/);
-#      $html_string = <$data>;
-#    }
-#    else {
-#      $html_string = $data;
-#    }
-#    my %te_parms = (
-#      headers => ['Splits:'],
-#      debug   => $self->{debug},
-#    );
-#    my $te = HTML::TableExtract->new(%te_parms);
-#    $te->parse($html_string);
-#    my $table = $te->first_table_found || return [];
-#    my($split_line) = grep(defined && /split/i, $table->hrow);
-#    $split_line =~ s/^\s*splits:?\s*//i;
-#    my @rows;
-#    foreach (grep(/\w+/, split(/\]\s*,\s+/, $split_line))) {
-#      s/\s+$//;
-#      next if /none/i;
-#      next unless s/\s*\[(\d+):(\d+).*$//;
-#      my($post, $pre) = ($1, $2);
-#      my $date = ParseDate($_)
-#        or croak "Problem parsing date string '$_'\n";
-#      push(@rows, [$date, $post, $pre]);
-#    }
-#    \@rows;
-#  };
-#}
-
 sub labels {
   my $self = shift;
   my %parms = @_;
   my $target_mode = $parms{target_mode} || $self->target_mode;
-  my $parse_mode  = $parms{parse_mode}  || $self->parse_mode;
   my @labels;
   if ($target_mode eq 'split') {
-    @labels = qw( date open );
-    $self->parse_mode('html');
+    @labels = qw( date stock );
   }
   else {
     @labels = $self->SUPER::labels(%parms);
@@ -135,43 +75,6 @@ sub labels {
   @labels;
 }
 
-sub extractors {
-  # Override. If we are pulling CSV, save some time by skipping
-  # extraction attempts
-  my $self  = shift;
-  my %parms = @_;
-  my $target_mode = $parms{target_mode} || $self->target_mode;
-  my $parse_mode  = $parms{parse_mode}  || $self->parse_mode;
-  if ($target_mode eq 'split') {
-    $parse_mode = 'html';
-    $self->parse_mode($parse_mode);
-  }
-  return () if $parse_mode eq 'csv';
-  my %extractors;
-  my $date_column = $self->label_column('date');
-  my $split_column = 1;
-  $extractors{'split'} = sub {
-    my $row = shift;
-    die "row as array ref required" unless ref $row;
-    # example split: "3 : 1 Stock Split"
-    my($post, $pre) = $row->[$split_column] =~ /(\d+)\s*:\s*(\d+).*Split/i;
-    return undef unless $post && $pre;
-    [ $row->[$date_column], $post, $pre ];
-  };
-  #my $div_column = 1;
-  #$extractors{dividend} = sub {
-  #  # Get a row as array ref, see if it contains dividend info. If so,
-  #  # return another array ref with the extracted info.
-  #  my $row = shift;
-  #  die "row as array ref required\n" unless ref $row;
-  #  # example dividend: "$0.01 Cash Dividend"
-  #  my($div) = $row->[$div_column] =~ /\$*(\d*\.\d+).*Dividend/i;
-  #  return undef unless defined $div;
-  #  [ $row->[$date_column], $row->[$div_column] ];
-  #};
-  %extractors;
-}
-
 sub url_maker {
   my($self, %parms) = @_;
   my $target_mode = $parms{target_mode} || $self->target_mode;
@@ -196,46 +99,44 @@ sub url_maker {
     ($start_date, $end_date) = ($end_date, $start_date);
   }
 
-  if ($target_mode eq 'split') {
-    $self->parse_mode($parse_mode = 'html');
-  }
-  else {
-    $self->parse_mode($parse_mode = 'csv');
-  }
-
-  my $base_url = $parse_mode eq 'csv' ? $self->url_base_csv() :
-                                        $self->url_base_html();
+  my $host = "query1.finance.yahoo.com";
+  my $base_url = "https://$host/v7/finance/download/$ticker?";
   my @base_parms;
   if ($start_date) {
     my($y, $m, $d) = $self->ymd($start_date);
-    $m = sprintf("%02d", $m - 1);
-    push(@base_parms, "a=$m", "b=$d", "c=$y");
+    my $ts = Date_SecsSince1970($m, $d, $y, 0, 0, 0);
+    push(@base_parms, "period1=$ts");
   }
   if ($end_date) {
     my($y, $m, $d) = $self->ymd($end_date);
-    $m = sprintf("%02d", $m - 1);
-    push(@base_parms, "d=$m", "e=$d", "f=$y");
+    my $ts = Date_SecsSince1970($m, $d, $y, 0, 0, 0);
+    $ts += 24*60*60;
+    push(@base_parms, "period2=$ts");
   }
-  my $g = $target_mode eq 'quote' ? $grain : 'v';
-  $ticker ||= 'BOOLEAN';
-  push(@base_parms, "g=$g", "s=$ticker");
-  
-  if ($parse_mode eq 'html') {
-    my $cursor = 0;
-    my $window = 66;
-    return sub {
-      my $url = $base_url . '?' .
-        join('&', @base_parms,
-                 "z=$window", "y=$cursor");
-      $cursor += $window;
-      $url;
-    }
+
+  my $interval = "1d";
+  if ($grain eq 'w') {
+    $interval = "1wk";
   }
-  else {
-    my @urls = $base_url .  '?' . join('&', @base_parms);
-    $urls[0] .= '&ignore=.csv' if $parse_mode eq 'csv';
-    return sub { pop @urls }
+  elsif ($grain eq 'm') {
+    $interval = "1mo";
   }
+  push(@base_parms, "interval=$interval");
+
+  if ($target_mode eq "quote") {
+    push(@base_parms, "events=history");
+  }
+  elsif ($target_mode eq "dividend") {
+    push(@base_parms, "events=div");
+  }
+  elsif ($target_mode eq "split") {
+    push(@base_parms, "events=split");
+  }
+
+  push(@base_parms, "crumb=" . $self->{crumb});
+
+  my @urls = $base_url . join('&', @base_parms);
+  return sub { pop @urls };
 }
 
 1;
